/* 
** geoffspline.c : Main source file for Mirage project geoffspline.
*/

#include "mirage.h"

/*********** Prototypes for geoffspline.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make_geoffspline(object *active_camera);

/************ Global variables ******************/
GeoffSpline *the_spline;



/*********** Function definitions, geoffspline.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv, "geoffspline.key");

  // Initialiase global variables for this project.
  init_globals();

  // Set the scene creator routine.
  set_worldmaker(make_geoffspline);

  // Create an animation.
  create_animation();
}

// Initialise global variables needed for the animation.
void init_globals(void)
{
  // Set up any globals here (load in texturemaps, splines, etc.).
  the_spline = new GeoffSpline("spline.txt");
}

object make_geoffspline(object *active_camera)
{
  object cameraobj, mycluster, scene, lightobj, lightobj2, ground;
  object test_obj2;
  Camera *theCamera;
  Light *theLight, *theLight2;
  double t;
  vector obj_pos, obj_fwd, obj_up;

  scene = new_nothing();

  // Point the camera along -Y, with the up-vector along +Z.
  cameraobj = create_default_camera(400, 300, 0, 20);
  *active_camera = cameraobj;
  cameraobj->rotate(X, Y, -90.0);
  cameraobj->shift(0.0, 5.0, 0.0);

  theLight = new PointLight(0.8, 0.8, 0.8);
  theLight2 = new AmbientLight(0.2, 0.2, 0.2);

  lightobj = Dagnode::newLight(theLight);
  lightobj->shift(-10.0, 10.0, 10.0);

  lightobj2 = Dagnode::newLight(theLight2);

  test_obj2 = new_sphere();
  test_obj2->stretch(0.125, 0.075, 0.2);
  test_obj2->material = new Material(new DiffuseComponent(1.0, 0.0, 0.0),
				     new PhongComponent(40.0,
							1.0, 1.0, 1.0));
  test_obj2->rotate(Z, X, 90.0);

  the_spline->get_position(obj_pos);
  the_spline->get_orientation(obj_fwd, obj_up);

  test_obj2->rotate(obj_fwd, obj_up);
  test_obj2->shift(obj_pos);

  ground = new_plane();
  ground->rotate(X, 180);
  ground->shift(0.0, 0.0, -1.0);
  ground->material = new Material(new DiffuseComponent(0.0, 0.85, 0.0));

  LINK(scene, PLUS, cameraobj);
  LINK(scene, PLUS, lightobj);
  LINK(scene, PLUS, lightobj2);
  LINK(scene, PLUS, ground);
  LINK(scene, PLUS, test_obj2);

  mycluster = new_stupid_cluster(bind(scene));

  Color::assign(background_color, 0.4, 0.1, 0.4);

  return mycluster;
}

