/* 
** imagetexture.c : Main source file for Mirage project imagetexture.
*/

#include "mirage.h"

/*********** Prototypes for imagetexture.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make_imagetexture(object *active_camera);
object make_picture(void);




/*********** Function definitions, imagetexture.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv, "imagetexture.key");

  // Initialiase global variables for this project.
  init_globals();

  // Set the scene creator routine.
  set_worldmaker(make_imagetexture);

  // Create an animation.
  create_animation();
}

// Initialise global variables needed for the animation.
void init_globals(void)
{
  // Set up any globals here (load in texturemaps, splines, etc.).
}

// This routine returns an object that is the scene.
object make_imagetexture(object *active_camera)
{
  object the_scene, the_camera, camera_focus;
  vector focus_position = {0, 0, 2};
  vector camera_position = {4, 8, 2};

  the_scene = new_nothing();

  // Add an object to look at in the scene.
  camera_focus = new_nothing();
  camera_focus->label("focus-label");
  LINK(the_scene, PLUS, shift(focus_position, bind(camera_focus)));

  // Create the camera.
  the_camera = new_camera(new PinholeOptics(20.0, 1),
			  new StandardRayTracer,
			  new Antialiaser(0),
			  new ColorStorage(400, 300),
			  NULL);
  *active_camera = the_camera;
  the_camera->label("camera-label");
  the_camera->affect = new PointAtAffect("camera-label", "focus-label");
  LINK(the_scene, PLUS, shift(camera_position, bind(the_camera)));

  // Add point and ambient lights.
  LINK(the_scene, PLUS, shift(+5000.0, +3000.0, +3000.0,
			      new_point_light(0.6, 0.6, 0.6)));
  LINK(the_scene, PLUS, new_ambient_light(0.4, 0.4, 0.4));

  // Add an object to look at.
  LINK(the_scene, PLUS, make_picture());

  Color::assign(background_color, 0.0, 0.0, 0.5);

  // Return the scene, with a cluster wrapped around it.
  return new_stupid_cluster(bind(the_scene));
}

object make_picture(void)
{
  object picture_frame;

  picture_frame = rotate(Z, Y, -90, new_plane());
  LINK(picture_frame, MINUS, shift(0, 0, 4, new_plane()));
  LINK(picture_frame, MINUS, reflect(Z, new_plane()));
  LINK(picture_frame, MINUS, shift(1, 0, 0, rotate(Z, X, 90, new_plane())));
  LINK(picture_frame, MINUS, shift(-1, 0, 0, rotate(Z, X, -90, new_plane())));
  
  picture_frame = bind(picture_frame); // bind is just like Katachi's new()

  picture_frame->material = white;
  
  // The parameters for the ImageTexture ... the first one is the precedence
  // of the texture, leave this 0 unless you are trying to overlay textures.
  // The next two sets of three specify where the X axis of the image is
  // mapped to and where the Y axis of the image is mapped to.
  //
  // Note that the block we are applying it to goes from X -1 - +1 and Z 0 - 4
  // and we want its edges to be untextured, so we apply the ImageTexture
  // to just within these boundaries.
  //
  // Note also that since the Y axis of any image increases as we go
  // _down_ the image, we map image_y = 0 to Z = 4 and image_y = height to
  // Z = 0.
  //
  // The other parameters ... may be SINGLE or TILED to say whether or not
  // the picture tiles, OPAQUE or TRANSPARENT to say how the image is applied,
  // and the next parameter is whether or not to use bilinear filtering (TRUE).
  // The last two parameters are flags that say whether or not to set the
  // specular and transparent components of the material to zero. This is
  // useful when using an ImageTexture to, for example, apply a label to
  // a glass bottle.
  picture_frame->texture = new ImageTexture(0, "tex.ppm",
					    X, -0.999, +0.999,
					    Z,  3.999,  0.001,
					    SINGLE, TEXMETHOD_OPAQUE, TRUE,
					    FALSE, FALSE);

  // Note that we must do this before we rotate the object any further so
  // as not to disturb where the texture goes on the object.
  return bind(picture_frame);
}
