/* 
** lensflare.c : Main source file for Mirage project lensflare.
**
** This is an example program to show you how to use the built-in lensflares
** that are defined in flare.c|h.
*/

#include "mirage.h"

/*********** Prototypes for lensflare.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make_lensflare(object *active_camera);

/*********** Function definitions, lensflare.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv, "lensflare.key");
  set_worldmaker(make_lensflare);
  create_animation();
}

// This routine returns an object that is the scene.
object make_lensflare(object *active_camera)
{
  object cameraobj, lightobj, scene;
  vector light_pos;
  Lensflare *the_flare;

  scene = new_nothing();

  cameraobj = create_default_camera(400, 300, 3, 20.0);

  *active_camera = cameraobj;

  LINK(scene, PLUS, cameraobj);

  // Create a light source that will give a lensflare.
  lightobj = shift(5.0, 0.5, 0.2, new_point_light(1.0, 1.0, 1.0));
  LINK(scene, PLUS, lightobj);

  // Black background so that we can see the lensflare properly.
  Color::assign(background_color, 0, 0, 0);

  // Create the Lensflare and add the light source's position. Could pass
  // a flare_parameters structure to the Lensflare, but this just uses the
  // default lensflare. See flare.c | flare.h for more details about
  // flare_parameters.
  the_flare = new Lensflare(cameraobj, 1);
  Vector::assign(light_pos, 5.0, 0.8, 0.1);
  the_flare->add_light(light_pos, 0.25, 1.0);

  // The foreground texture applies the lensflare.
  foreground_texture = new FlareForeground(the_flare);

  return new_stupid_cluster(bind(scene));
}
