/* 
** pointat.c : Main source file for Mirage project pointat.
*/

#include "mirage.h"

/*********** Prototypes for pointat.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make_pointat(object *active_camera);

/*********** Function definitions, pointat.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv);

  // Initialiase global variables for this project.
  init_globals();

  // Set the scene creator routine.
  set_worldmaker(make_pointat);

  // Create an animation.
  create_animation();
}

// Initialise global variables needed for the animation.
void init_globals(void)
{
  // Set up any globals here (load in texturemaps, splines, etc.).
}

// This routine returns an object that is the scene.
object make_pointat(object *active_camera)
{
  object mycluster, cameraobj, lightobj, lightobj2;
  object sphere_obj, cyl_obj, ground, scene;

  scene = new_nothing();

  // Point the camera along -Y, with the up-vector along +Z.
  cameraobj = create_default_camera(400, 300, 0, 20);
  *active_camera = cameraobj;

  lightobj = shift(0.0, -1.5, 0.0, new_point_light(0.8, 0.8, 0.8));
  lightobj2 = new_ambient_light(0.2, 0.2, 0.2);

  cyl_obj = new_unitcyl();
  cyl_obj->stretch(0.3, 0.3, 1.0);
  cyl_obj->rotate(Z, X, 90.0);
  cyl_obj->material = new Material(new DiffuseComponent(0.5, 0.0, 0.0),
				   new SpecTransComponent(1.5,
							  0.5, 0.5, 0.5,
							  0.0, 0.0, 0.0));
  // Label the thing to do the pointing. Note that the cylinder is pointing
  // along +X currently, with its up-vector along +Z.
  cyl_obj->label("wand");

  // Tell the cylinder to point at the thing labeled "ball".
  cyl_obj->affect = new PointAtAffect("wand", "ball");

  // Shift the cylinder (pointing in the right direction) accordingly.
  cyl_obj = bind(cyl_obj);
  cyl_obj->shift(0.0, -2.5, 0.0);

  // Create the "ball" object and place it (moving from right to left).
  sphere_obj = new_sphere();
  sphere_obj->stretch(0.3, 0.3, 0.3);
  sphere_obj->material = white;
  sphere_obj->label("ball");
  sphere_obj = bind(sphere_obj);
  sphere_obj->shift(2.0 - 4 * worldtime, -3.5, 1.5);

  ground = new_plane();
  ground->rotate(Z, X, 180.0);
  ground->shift(0.0, 0.0, -1.0);
  ground->material = new Material(new DiffuseComponent(0.0, 0.8, 0.0),
				  new SpecTransComponent(1.5,
							 0.2, 0.2, 0.2,
							 0.0, 0.0, 0.0));

  scene = new_nothing();
  LINK(scene, PLUS, shift(0, 5, 0, rotate(X, Y, -90, cameraobj)));
  LINK(scene, PLUS, lightobj);
  LINK(scene, PLUS, lightobj2);
  LINK(scene, PLUS, cyl_obj);
  LINK(scene, PLUS, sphere_obj);
  LINK(scene, PLUS, ground);
  mycluster = new_stupid_cluster(bind(scene));

  Color::assign(background_color, 0.4, 0.1, 0.4);

  return mycluster;
}


