/* 
** surface-distortion.c : Main source file for Mirage project surface-distortion.
*/

#include "mirage.h"

/*********** Prototypes for surface-distortion.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make_surface_distortion(object *active_camera);
void wrinkle_distortion(primnode *the_node, transform dist_transform,
			vector normal, double *t_val, vector ray_from,
			vector ray_dir, uint ray_flags);

/*********** Function definitions, surface-distortion.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv, "surface-distortion.key");

  // Initialiase global variables for this project.
  init_globals();

  // Set the scene creator routine.
  set_worldmaker(make_surface_distortion);

  // Create an animation.
  create_animation();
}

// Initialise global variables needed for the animation.
void init_globals(void)
{
  // Set up any globals here (load in texturemaps, splines, etc.).
}

// This routine returns an object that is the scene.
object make_surface_distortion(object *active_camera)
{
  object cameraobj, mycluster, scene, lightobj, lightobj2, ground;
  object wrinkly_obj, sub_obj, plus_obj;

  scene = new_nothing();

  // Point the camera along -Y, with the up-vector along +Z.
  cameraobj = create_default_camera(400, 300, 0, 20);

  *active_camera = cameraobj;

  lightobj = shift(-10.0, 10.0, 10.0, new_point_light(0.8, 0.8, 0.8));
  lightobj2 = new_ambient_light(0.2, 0.2, 0.2);

  wrinkly_obj = new_cube();
  wrinkly_obj->stretch(0.6, 0.6, 0.6);
  wrinkly_obj->material = new Material(new DiffuseComponent(0.7, 0.7, 0.9),
				       new SpecTransComponent(1.0,
							      0.1, 0.1, 0.1,
							      0.0, 0.0, 0.0));
  wrinkly_obj->surface_distortion = wrinkle_distortion;

  ground = new_plane();
  ground->rotate(X, 180);
  ground->shift(0.0, 0.0, -1.0);
  ground->material = new Material(new DiffuseComponent(0.0, 0.8, 0.0),
				  new SpecTransComponent(1.5,
							 0.2, 0.2, 0.2,
							 0.0, 0.0, 0.0));

  sub_obj = wrinkly_obj->bind();
  sub_obj->rotate(Z, 45.0);
  sub_obj->shift(0.0, -1.5, -0.5);

  plus_obj = wrinkly_obj->bind();
  plus_obj->shift(0.0, -3.5, 0.0);

  ground = ground->link(MINUS, sub_obj);
  ground = ground->bind();

  scene = scene->link(PLUS, shift(0, 5, 0, rotate(X, Y, -90, cameraobj)));
  scene = scene->link(PLUS, lightobj);
  scene = scene->link(PLUS, lightobj2);
  scene = scene->link(PLUS, ground);
  scene = scene->link(PLUS, plus_obj);
  scene = scene->bind();

  mycluster = new_stupid_cluster(scene);

  Color::assign(background_color, 0.4, 0.1, 0.4);

  return mycluster;
}

void wrinkle_distortion(primnode *the_node, transform dist_transform,
			vector normal, double *t_val, vector ray_from,
			vector ray_dir, uint ray_flags)
{
  vector prim_hit_point, prim_normal;
  vector noise_val, noise_sample;
  
  // Construct the worldspace hitpoint.
  Vector::mult(*t_val, ray_dir, prim_hit_point);
  Vector::add(ray_from, prim_hit_point, prim_hit_point);

  // Transform the hitpoint from worldspace into the primitivespace of the
  // primitive where the function was applied.
  Matrix::apply(dist_transform[AFT], prim_hit_point, prim_hit_point);

  // We currently have the normal in the primitivespace of the bottom-level
  // primitive that was hit. We want it in the primitivespace of the
  // primitive where the function was applied, so take it there (going through
  // worldspace along the way).

  // Go to worldspace from the primitivespace of the bottom-level primitive.
  Matrix::apply(the_node->position[FORE], normal, prim_normal);
  // Go to the primitivespace of the level where the function is applied.
  Matrix::apply(dist_transform[AFT], prim_normal, prim_normal);

  // Change the normal.
  Vector::mult(6.0, prim_hit_point, noise_sample);
  noise_3grad(noise_sample, noise_val);

  Vector::mult(0.6, prim_normal, prim_normal);
  Vector::mult(0.4, noise_val, noise_val);
  Vector::add(noise_val, prim_normal, prim_normal);

  // Take the normal back into the space that it was in originally.
  Matrix::apply(dist_transform[FORE], prim_normal, prim_normal);
  Matrix::apply(the_node->position[AFT], prim_normal, normal);
}



