/* 
** __PROJECT.c : Main source file for Mirage project __PROJECT.
*/

#include "mirage.h"

/*********** Prototypes for __PROJECT.c ************/

int main(int argc, char **argv);
void init_globals(void);
object make___PROJECT(object *active_camera);




/*********** Function definitions, __PROJECT.c *********/

// Initialise Mirage, initialise global variables and create an animation.
int main(int argc, char **argv)
{
  // Set up Mirage using the command line parameters.
  init_system(argc, argv, "__PROJECT.key");

  // Initialiase global variables for this project.
  init_globals();

  // Set the scene creator routine.
  set_worldmaker(make___PROJECT);

  // Create an animation.
  create_animation();
}

// Initialise global variables needed for the animation.
void init_globals(void)
{
  // Set up any globals here (load in texturemaps, splines, etc.).
}

// This routine returns an object that is the scene.
object make___PROJECT(object *active_camera)
{
  object the_scene = new_nothing(), the_camera, camera_focus;
  object the_object;
  vector focus_position = {0, 0, 0};
  vector camera_position = {4, 3, 0};

  // Add an object for the camera to point at. Provides an intuitive
  // way of positioning and orienting the camera.
  camera_focus = new_nothing();
  camera_focus->label("focus-label");
  LINK(the_scene, PLUS, shift(focus_position, bind(camera_focus)));

  // Create the camera. Image size may be modified by the parameter
  // picture-size in the keyfile __PROJECT.key.
  the_camera = new_camera(new PinholeOptics(20.0, 1),
			  new StandardRayTracer,
			  new Antialiaser(0),
			  new ColorStorage,
			  NULL);
  the_camera->label("camera-label");
  the_camera->affect = new PointAtAffect("camera-label", "focus-label");
  LINK(the_scene, PLUS, shift(camera_position, bind(the_camera)));

  // Add point and ambient lights.
  LINK(the_scene, PLUS, shift(+5000.0, +0.0, +3000.0,
			      new_point_light(0.6, 0.6, 0.6)));
  LINK(the_scene, PLUS, new_ambient_light(0.4, 0.4, 0.4));

  // Add an object to look at.
  the_object = new_sphere();
  the_object->material = new Material(new DiffuseComponent(0.1, 0.3, 0.075),
				      new PhongComponent(40.0, 1.0, 1.0, 1.0));
  LINK(the_scene, PLUS, stretch(1.2, 1.0, 0.8, bind(the_object)));

  *active_camera = the_camera;

  // Set the background colour.
  Color::assign(background_color, 0.6, 0.6, 0.6);

  // Return the scene, with a cluster wrapped around it.
  return new_stupid_cluster(bind(the_scene));
}

