/* globals.h : Global variables
** 
** This file holds 'extern' definitions for all of the global variables used
** in Mirage.
**
** Chris Butcher, 27 November 1996
*/

// The version of the Mirage library.
extern char mirage_version_string[MIRAGE_STRING_LENGTH];

// Flag to control whether debugging warnings are printed.
extern int debugging_messages;

// Current time in worldunits (seconds usually)
extern double worldtime;

// Number of frames per worldunit (seconds ...)
extern double anim_fps;

// Timestep per frame in worldunits (seconds usually)
extern double anim_timestep;

// Current frame number for output.
extern int anim_framenumber;

// Final frame to render.
extern int anim_lastframe;

// Step size in frames (5 = every fifth frame, etc)
extern int anim_frameskip;

// The function that is called to generate the world for rendering.
// Set by the user by calling set_worldmaker().
extern object (*worldmaker)(object *active_camera);

// A UNIX command called when each frame is generated
extern char postcommand[MIRAGE_STRING_LENGTH];

// The file type to write
extern filetype output_filetype;

// The global label list used for storing all labels that are attached
// to the scene being rendered.
extern LabelList *world_label_list;

// The background colour (applied if there is no background_texture).
extern color background_color;

// The foreground_ and background_texture objects are set by the user.
// If not set, they are NULL and so are not applied.
extern BackgroundTexture *foreground_texture, *background_texture;

// The global list of all lights.
extern lightList globalLightList;

// Maximum depth of secondary rays to cast from a given primary ray.
extern int maximum_secondary_ray_depth;

// Ray counting variables.
extern int counter_primary_rays, counter_secondary_rays, counter_shadow_rays;
extern int counter_initial_rays, counter_antialiasing_rays;
extern int display_ray_statistics;

// Flag to say whether or not to display progress of raytracing.
extern int show_raytracing_progress;

// The name of the rendering project.
extern char project_name[MIRAGE_STRING_LENGTH];

// A vector that points along +Z for use as a default up-vector.
extern vector plus_Z;

// A zero vector that can be referenced by clients.
extern vector zero_vector;

// The quaternion multiplicative identity.
extern quaternion identity_quaternion;

// A flag to say whether or not newly allocated dynamic memory should be
// deallocated at the end of each frame. This is FALSE initially, but is
// set to TRUE once the call to create_animation() is made. Thus you must
// allocate any memory that you want for the entire animation *before*
// you call create_animation.
extern int insert_dynamic_memory_into_deletion_list;

// A debugging flag to say whether every pixel that the PixelSampler
// raytraces should be printed out on stdout.
extern int pixel_sampler_print_every_pixel;

// A flag to say whether or not textures should be applied.
extern int apply_textures_to_objects;

// A flag to say whether or not objects are drawn as their base color only.
extern int objects_color_only;

// A flag to say whether or not the memory statistics are displayed.
extern int display_memory_messages;

// A flag to say whether Mirage should be totally silent.
extern int completely_silent;

// RCSG backtraversal array.
extern const int backtraverse_size;
extern const char backtraverse[];

// The point-contents that are everywhere that objects aren't.
extern PointInfo *ether;

// Standard material library.
extern Material *black, *dkgrey, *grey, *ltgrey, *white,
                *ltred, *ltgreen, *ltblue,
		*red, *green, *blue,
		*dkred, *dkgreen, *dkblue,
                *magenta, *cyan, *yellow,
                *lavender, *purple,
		*orange, *ltbrown, *brown, *dkbrown, *terracotta,
                *transp, *perspex, *glass, *clear,
                *mirror, *gold, *silver,
                *china, *default_material;

