/* label.h : Header file for the global label list.
** 
** This file defines the Label struct and LabelList class.
**
** Chris Butcher, 20 December 1996
*/

// The Label struct holds the data relating to a label that is
// attached to a specific point in the dag-graph.
// It holds the name of the label, and the dagnode where it was
// placed.
typedef struct Label {
  // The name of the label.
  char label_name[MIRAGE_STRING_LENGTH];

  // The dagnode where the label was attached.
  object label_place;

  // The transform specifying this object's position in the
  // uppermost world-space.
  transform label_transform;

  // Has this label's position been set yet?
  int label_position_set;
} label;

// The LabelList class stores all of the labels attached to
// objects in the current scene. Labels are added to this list
// by the user and may be retrieved at any stage (usually during
// the affect routines, which are routines that modify the dag-
// graph just before it is converted into RCSG form).
class LabelList : public DeletableObject {
public:
  label *label_storage;
  int list_size;

  // next_free is the index of the next free label storage in the list.
  int next_free;

  // Clear the list.
  void init(void);

  // Insert a label into the list.
  void insert(char *newname, object newplace);

  // Retrieve a label from the list. Returns TRUE on success.
  int find(char *findname, object *findplace, transform findtrans);

  // Set the position of a label in the list.
  void set_transform(object the_obj, transform obj_position);

  // Constructor.
  LabelList(int my_size);

  // Destructor deallocates the memory allocated in this frame.
  virtual ~LabelList(void) {
    free(label_storage);
  }
};






