/* macros.h : #define macros that are used in Mirage.
** 
** All of the #define macros that are needed by Mirage are found here,
** except those that are needed only in one specific place.
**
** Chris Butcher, 27 November 1996
*/

#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a, b) ((a) < (b) ? (b) : (a))
#endif

#define MIN3(a, b, c) ((a) < (b) ? ((a) < (c) ? (a) : (c)) \
                                 : ((b) < (c) ? (b) : (c)))
#define MAX3(a, b, c) ((a) > (b) ? ((a) > (c) ? (a) : (c)) \
                                 : ((b) > (c) ? (b) : (c)))

#define SQR(x)      ((x) * (x))
#define CUBE(x)     ((x) * (x) * (x))

// Check to see if the value (x) is too small. If |x| < TINY, we replace it
// with the equivalent +TINY or -TINY (keep the sign the same). Also warn
// the user that we've done so.
void zerowarn(char *location);
#define CHECK0(x, msg) ((((x) > TINY) || (-(x) > TINY)) ? (x) : (zerowarn(msg), (x) < 0.0 ? -TINY : TINY))


// Convert degrees to radians and vice versa.
#define DEG2RAD(x)  ((x) * PI / 180.0)
#define RAD2DEG(x)  ((x) * 180.0 / PI)

// A macro to clamp a value between two bounds.
#define CLAMP(a, x, b)    ((x) = ((x) < (a)) ? (a) : (((x) > (b)) ? (b) : (x)))

// Macro for dag-graph construction.
#define LINK(a, m, b)   a = a->link(m, b);

// The sign of a number.
#define SIGN(x) (((x) == 0) ? 0 : (((x) < 0) ? -1 : +1))



