/* memory.h : Definitions for the working memory manager.
** 
** This file has definitions for the working memory manager.
**
** Chris Butcher, 28 December 1996
*/

// A DeletableObject is the root class for all dynamically allocated objects
// so that they can be stored as (DeletableObject *) in the dynamic memory
// list.
class DeletableObject {
public:

  // Flag this object for deletion at the end of the current frame.
  void mark_for_deletion(void);

  // Deallocate this object and its dynamically allocated memory.
  // This is actually done in all of the subclasses, but this stub
  // is necessary for the parent class.
  virtual ~DeletableObject(void) {
  }
};

// Routines for managing the working memory store.

// Declare that all of these things are classes, inside namespace Mirage.
class Dagnode;
class IntersectionList;
class Material;
class TextureCollection;

void init_working_memory(void);
void shutdown_working_memory(void);
void frame_init_working_memory(void);
void frame_shutdown_working_memory(void);
Dagnode *new_dag(void);
Dagnode *new_permanent_dag(void);
struct PrimitiveNode *new_primnode(unsigned int *new_thing);
struct octreenode *new_octree(unsigned int *new_thing);
unsigned int new_rcsg(unsigned int left, unsigned int right,
		      linktype mymode, int num_prims);
IntersectionList *new_intersection_list(void);
struct Intersection *alloc_intersection(void);
struct Intersection *new_intersection(uint hit_flags,
				      struct PrimitiveNode *node, vector norm,
				      double t_value,
				      int from_empty, int to_empty,
				      Material *mat_from,
				      Material *mat_to,
				      SfxType sfxtype1, SfxType sfxtype2,
				      void *sfx1, void *sfx2,
				      int multitex1, int multitex2,
				      void *tex1, void *tex2,
				      vector ray_from, vector ray_dir, 
				      uint ray_flags,
				      struct PrimitiveNode *real_primnode,
				      struct PrimitiveNode *validate_primnode);
struct _SingleTexture *alloc_texture(void);
TextureCollection *alloc_textureCollection(void);
struct PrimitiveNode *retrieve_primnode(unsigned int index);
struct octreenode *retrieve_octree(unsigned int index);
struct RCSGnode *retrieve_rcsgnode(unsigned int index);
void free_intersection_list(class IntersectionList *the_list);
void free_octreenode(unsigned int the_thing);
void free_rcsgnode(unsigned int the_thing);
void free_tree(unsigned int the_thing);
void free_intersection(struct Intersection *the_int);


