/* mirage.h : Main header file for Mirage.
** 
** From this file all the macros, constants, inline functions and
** prototypes that may be needed by a client are included.
**
** This file should be included in every client that uses the system.
**
** Chris Butcher, 27 November 1996
*/

#ifndef INCLUDE_MIRAGE
#define INCLUDE_MIRAGE

// Note that this is the version string of Mirage's latest minor version
// update. Bugfix updates (1.0.4.3, 1.0.4.4 etc) are not included here.
// Details are given in $MIRAGE/CHANGELOG though.
#define MIRAGE_VERSION_STRING         "1.0.6"

// MIRAGE: version 1.0.1           release date 10/03/97
// MIRAGE: version 1.0.2           release date 08/04/97
// MIRAGE: version 1.0.3           release date 15/05/97
// MIRAGE: version 1.0.4           release date 28/08/97
// MIRAGE: version 1.0.5           release date 02/02/98
// MIRAGE: version 1.0.6           release date 19/05/98

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <time.h>
#include <stdarg.h>
#ifndef _WIN32
#include <unistd.h>
#else
#include <afxwin.h>
#endif

// By default Mirage compiles on a UN*X machine.
#define UNIX_SPECIFIC

// Which endian is our machine?
#ifdef macintosh
#ifndef BIG_ENDIAN
#define BIG_ENDIAN
#endif
#else
#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN
#endif
#endif

// If we are compiling on a Macintosh we must use our own drand48
// and cbrt code.
#ifdef macintosh
#define USE_OWN_DRAND48
#define USE_OWN_CBRT
#undef UNIX_SPECIFIC
#endif

// If we are compiling under win32 we must use our own drand48
// and cbrt code. We also use namespaces so as not to conflict
// with the MFC definitions. And we have to undefine those asinine
// global min(x, y) and max(x, y) MACROS!!! What kind of programmer
// uses lower-case three-letter macros?!?! Don't forget about IN and
// OUT!!!
#ifdef _WIN32
#define USE_OWN_DRAND48
#define USE_OWN_CBRT
#undef UNIX_SPECIFIC
#define USE_NAMESPACES

#ifdef min
#undef min
#endif
#ifdef max
#undef max
#endif
#ifdef IN
#undef IN
#endif
#ifdef OUT
#undef OUT
#endif

#endif  // _WIN32

// If the UNDEFINE_LIMITS compile option is used then DBL_MAX, DBL_MIN,
// FLT_MAX and FLT_MIN are #undefined before limits.h to avoid compiler
// warning messages on some SGI Indigo systems about their redefinition.
#ifdef UNDEFINE_LIMITS

#ifdef DBL_MAX
#undef DBL_MAX
#endif

#ifdef DBL_MIN
#undef DBL_MIN
#endif

#ifdef FLT_MAX
#undef FLT_MAX
#endif

#ifdef FLT_MIN
#undef FLT_MIN
#endif

#endif // UNDEFINE_LIMITS

#include <limits.h>

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

#ifndef _WIN32
#include "assert.h"
#endif
#include "constants.h"
#include "typedefs.h"
#include "macros.h"
#include "vectors.h"
#include "matrix.h"
#include "quaternion.h"
#include "memory.h"
#include "color.h"
#include "colorvalue.h"
#include "light.h"
#include "subdivide.h"
#include "material.h"
#include "dagnode.h"
#include "texture.h"
#include "rand48.h"
#include "gaseous.h"
#include "camera.h"
#include "raytrace.h"
#include "list.h"
#include "label.h"
#include "blockqueue.h"
#include "xwindows.h"
#include "antialias.h"
#include "flare.h"
#include "pixmap.h"
#include "imagemap.h"
#include "path.h"
#include "primitives.h"
#include "heightfield.h"
#include "volume.h"
#include "prim_trimesh.h"
#include "globals.h"
#include "prototypes.h"
#include "useful.h"
#include "userinterface.h"

#ifdef USE_NAMESPACES
}; // namespace Mirage {
#endif

#endif    // INCLUDE_MIRAGE
