/* prim_full.c : The 'full' primitive
** 
** The full primitive is simply full space; all points are inside.
**
** Chris Butcher, 28 November 1996
*/

#include "mirage.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

int fullPrimitive::ray_intersect(IntersectionList *the_list, primnode *node,
				 primnode *validate_node, 
				 vector ray_from, vector ray_dir,
				 uint ray_flags, signature ray_signature)
{
  // Full space has no surfaces, so no rays ever intersect a surface.
  return FALSE;
}

int fullPrimitive::ray_intersect(vector ray_from, vector ray_dir,
				 rayintersect *int_storage)
{
  // Full space has no surfaces, so no rays ever intersect a surface.
  return 0;
}

intersect_result fullPrimitive::point_intersect(primnode *node, vector a_point,
						PointInfo *info,
						uint ray_flags)
{
  // All points are IN full space.
  set_point_contents(node, info);
  return IN;
}

intersect_result fullPrimitive::point_intersect(vector a_point)
{
  // All points are IN full space.
  return IN;
}

intersect_result fullPrimitive::voxel_intersect(box v)
{
  // All voxels are IN full space.
  return IN;
}

char *fullPrimitive::prim_name(void) {
  return "Full";
}


#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
