/* prim_nothing.c : The 'nothing' primitive
** 
** The nothing primitive is simply empty space; useful only for
** attaching lights to and as a tool for easily-readable code.
**
** Chris Butcher, 3 December 1996
*/

#include "mirage.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

int nothingPrimitive::ray_intersect(IntersectionList *the_list,
				    primnode *node, primnode *validate_node, 
				    vector ray_from,
				    vector ray_dir, uint ray_flags,
				    signature ray_signature)
{
  // No surfaces in nothing, so no rays ever intersect a surface.
  return FALSE;
}

int nothingPrimitive::ray_intersect(vector ray_from, vector ray_dir,
				    rayintersect *int_storage)
{
  // Empty space has no surfaces, so no rays ever intersect a surface.
  return 0;
}

intersect_result nothingPrimitive::point_intersect(primnode *node,
						   vector a_point,
						   PointInfo *info,
						   uint ray_flags)
{
  return OUT;
}

intersect_result nothingPrimitive::point_intersect(vector a_point)
{
  // All points are OUT; nothing to be in.
  return OUT;
}

intersect_result nothingPrimitive::voxel_intersect(box v)
{
  // All voxels are OUT as it is empty space.
  return OUT;
}

char *nothingPrimitive::prim_name(void) {
  return "Nothing";
}


#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
