/* rand48.c : Code for the rand48 routines.
** 
** This file is the Mirage implementation of the rand48 routines that
** were written by Martin Birgmeier.
**
** Chris Butcher, 16 September 1997
*/

#include "mirage.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

#ifdef USE_OWN_DRAND48

// Mirage prefers to use the rand48 routines provided with the system.
// However if they aren't there, it provides its own.

/*
 * Copyright (c) 1993 Martin Birgmeier
 * All rights reserved.
 *
 * You may redistribute unmodified or modified versions of this source
 * code provided that the above copyright notice and this and the
 * following conditions are retained.
 *
 * This software is provided ``as is'', and comes with no warranties
 * of any kind. I shall in no event be liable for anything that happens
 * to anyone/anything when using this software.
 */

/*
 * Global variables.
 */

unsigned short __mirage_rand48_seed[3] = {
  MIRAGE_RAND48_SEED_0,
  MIRAGE_RAND48_SEED_1,
  MIRAGE_RAND48_SEED_2
};

unsigned short __mirage_rand48_mult[3] = {
  MIRAGE_RAND48_MULT_0,
  MIRAGE_RAND48_MULT_1,
  MIRAGE_RAND48_MULT_2
};

unsigned short __mirage_rand48_add = MIRAGE_RAND48_ADD;

/*
 * Routines.
 */

void srand48(long seed)
{
  __mirage_rand48_seed[0] = MIRAGE_RAND48_SEED_0;
  __mirage_rand48_seed[1] = (unsigned short) seed;
  __mirage_rand48_seed[2] = (unsigned short) (seed >> 16);
  __mirage_rand48_mult[0] = MIRAGE_RAND48_MULT_0;
  __mirage_rand48_mult[1] = MIRAGE_RAND48_MULT_1;
  __mirage_rand48_mult[2] = MIRAGE_RAND48_MULT_2;
  __mirage_rand48_add     = MIRAGE_RAND48_ADD;
}

unsigned short *seed48(unsigned short xseed[3])
{
  static unsigned short sseed[3];
  
  sseed[0] = __mirage_rand48_seed[0];
  sseed[1] = __mirage_rand48_seed[1];
  sseed[2] = __mirage_rand48_seed[2];
  __mirage_rand48_seed[0] = xseed[0];
  __mirage_rand48_seed[1] = xseed[1];
  __mirage_rand48_seed[2] = xseed[2];
  __mirage_rand48_mult[0] = MIRAGE_RAND48_MULT_0;
  __mirage_rand48_mult[1] = MIRAGE_RAND48_MULT_1;
  __mirage_rand48_mult[2] = MIRAGE_RAND48_MULT_2;
  __mirage_rand48_add     = MIRAGE_RAND48_ADD;

  return sseed;
}

long nrand48(unsigned short xseed[3])
{
  __mirage_dorand48(xseed);

  return ((long) xseed[2] << 15) + ((long) xseed[1] >> 1);
}

long mrand48(void)
{
  __mirage_dorand48(__mirage_rand48_seed);

  return ((long) __mirage_rand48_seed[2] << 16)
    + (long) __mirage_rand48_seed[1];
}

long lrand48(void)
{
  __mirage_dorand48(__mirage_rand48_seed);

  return ((long) __mirage_rand48_seed[2] << 15)
    + ((long) __mirage_rand48_seed[1] >> 1);
}

void lcong48(unsigned short p[7])
{
  __mirage_rand48_seed[0] = p[0];
  __mirage_rand48_seed[1] = p[1];
  __mirage_rand48_seed[2] = p[2];
  __mirage_rand48_mult[0] = p[3];
  __mirage_rand48_mult[1] = p[4];
  __mirage_rand48_mult[2] = p[5];
  __mirage_rand48_add     = p[6];
}

long jrand48(unsigned short xseed[3])
{
  __mirage_dorand48(xseed);

  return ((long) xseed[2] << 16) + (long) xseed[1];
}

double erand48(unsigned short xseed[3])
{
  __mirage_dorand48(xseed);
  return ldexp((double) xseed[0], -48) +
         ldexp((double) xseed[1], -32) +
         ldexp((double) xseed[2], -16);
}

double drand48(void)
{
  return erand48(__mirage_rand48_seed);
}

void __mirage_dorand48(unsigned short xseed[3])
{
  unsigned long accu;
  unsigned short temp[2];
  
  accu = (unsigned long) __mirage_rand48_mult[0] * (unsigned long) xseed[0]
    + (unsigned long) __mirage_rand48_add;
  temp[0] = (unsigned short) accu;	/* lower 16 bits */

  accu >>= sizeof(unsigned short) * 8;
  accu += (unsigned long) __mirage_rand48_mult[0] * (unsigned long) xseed[1]
        + (unsigned long) __mirage_rand48_mult[1] * (unsigned long) xseed[0];
  temp[1] = (unsigned short) accu;	/* middle 16 bits */

  accu >>= sizeof(unsigned short) * 8;
  accu += __mirage_rand48_mult[0] * xseed[2]
    + __mirage_rand48_mult[1] * xseed[1]
    + __mirage_rand48_mult[2] * xseed[0];
  
  xseed[0] = temp[0];
  xseed[1] = temp[1];
  xseed[2] = (unsigned short) accu;
}

#endif // USE_OWN_DRAND48

#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
