/* scene.c : preparations done to the scene before rendering
** 
** This file defines the functions that prepare the scene to be rendered:
**   those to find labels, traverse the DAG and subdivide the clusters.
**
** Chris Butcher, 5 December 1996
*/

#include "mirage.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

// The global list of all lights.
lightList globalLightList;

// Traverse the DAG looking for the absolute locations of labels.
void scene_find_labels(object theScene)
{
  transform world_transform;

  Transform::unit(world_transform);
  recursive_find_labels(theScene, world_transform);
}

// Convert the DAG into an RCSG and account for all elastic links.
// Find all cameras' and lights' position matrices.
// Find the active camera.
void scene_create_RCSG(object theScene, primnode *convertedScene)
{
  // The world_transform of the top-level cluster is the unit transform.
  transform dummy_trans;

  // Convert the toplevel DAG structure [which must be a single primitive]
  // into a primitive node.
  if (((theScene->flags & FLAGS_PRIM) == 0) || (theScene->linkptr != NULL))
    fatal_error("scene_create_RCSG", "world_object not a primitive!",
		"The object that is returned by your worldmaker\n"
		"procedure must be a primitive. This means that you\n"
		"must wrap some kind of cluster object around the\n"
		"outside of your scene.\n"
		"e.g. new_stupid_cluster(bind(scene)).");

  convertedScene->flags = theScene->flags & FLAGS_NOTYPE;
  Transform::set(convertedScene->position, theScene->position);
  convertedScene->material = theScene->material;
  convertedScene->prim = (Primitive *) theScene->details;
  convertedScene->lightlist = &globalLightList;

  // Give the world_object a new TextureCollection with just the
  // texture of theScene in it (at the right place).
  if (theScene->texture != NULL)
    convertedScene->texcol = new_textureCollection(theScene->texture,
						   theScene->position);
  else
    convertedScene->texcol = NULL;

  // Haven't found any lights yet.
  globalLightList.init();

  // Traverse the top-level primitive defined in the DAG if it is a
  // cluster (if it isn't then the scene only consists of one primitive!)
  if (convertedScene->prim->is_cluster) {
    Transform::unit(dummy_trans);
    ((ClusterPrimitive *) convertedScene->prim)->create_RCSG(convertedScene,
							     dummy_trans);
  }
}

// Subdivide the clusters in the scene as necessary, dividing the RCSG tree.
void scene_subdivide(primnode *theScene)
{
  if (theScene->prim->is_cluster)
    ((ClusterPrimitive *) theScene->prim)->subdivide();
}


#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
