/* useful.h : Useful functions for scenes [prototypes and definitions]
** 
** This file holds all of the function prototypes and definitions for
** the collection of useful functions stored in useful.c.
**
** Chris Butcher, 15 January 1997
*/

// Convert a vector into spherical coordinates.
void spherical_coordinates(vector v, double *rho, double *theta, double *phi);

// Interpolates a value from lowval to highval by fraction (from 0 - 1)
// using the Geoff (cubic) function 3t^2 - 2t^3.
double cubicInterpolate(double lowval, double highval, double fraction);

// Returns the position of the highest bit set in an unsigned long as
// an integer (0 - 31), or -1 if it is zero.
int highest_bit(unsigned long theLong);

// The PointAtAffect is used for pointing one labeled object at another labeled
// object. It may also be given an up-vector.
class PointAtAffect : public Affect {
  char fromname[MIRAGE_STRING_LENGTH];
  char toname[MIRAGE_STRING_LENGTH];
  vector up_vector;
public:

  PointAtAffect(char *from_name, char *to_name) {
    strcpy(fromname, from_name);
    strcpy(toname, to_name);
    Vector::assign(up_vector, 0, 0, 1);
    mark_for_deletion();
  }

  PointAtAffect(char *from_name, char *to_name, vector my_up) {
    strcpy(fromname, from_name);
    strcpy(toname, to_name);
    Vector::set(up_vector, my_up);
    mark_for_deletion();
  }

  PointAtAffect(char *from_name, char *to_name,
		double up_x, double up_y, double up_z) {
    strcpy(fromname, from_name);
    strcpy(toname, to_name);
    Vector::assign(up_vector, up_x, up_y, up_z);
    mark_for_deletion();
  }

  // Apply the affect procedure.
  virtual void affect_object(object the_obj);

  virtual ~PointAtAffect() {
  }
};

// The HugHeightfieldAffect is used for making objects hug the surface
// of a heightfield.
//
// NB: this must ONLY be applied with the second label parameter as a
// Heightfield. It should be an object returned from
// new_heightfield(...) or Dagnode::newPrimitive(new Heightfield(...)).
class HugHeightfieldAffect : public Affect {
public:
  char objname[MIRAGE_STRING_LENGTH];
  char hfname[MIRAGE_STRING_LENGTH];

  HugHeightfieldAffect(char *obj_name, char *hf_name) {
    strcpy(objname, obj_name);
    strcpy(hfname,  hf_name);
    mark_for_deletion();
  }

  // Apply the affect procedure.
  virtual void affect_object(object the_obj);

  virtual ~HugHeightfieldAffect() {
  }
};

// The Noisetexture merely varies the diffuse colour of the surface by the
// specified amount at the specified scale using three-dimensional noise.
class Noisetexture : public Texture {
public:
  double scalefactor, noise_amount;

  // Apply the texture.
  void apply(vector ray_from, vector ray_dir, uint ray_flags,
	     Material *properties, vector hit_point, vector hit_normal,
	     RayTracer *engine, transform tex_transform);

  Noisetexture(double scale, double amt, int prec) {
    scalefactor = scale;
    noise_amount = amt;
    precedence = prec;

    mark_for_deletion();
  }

  void print_tex(void) {
    printf("Noisetexture: scale %f, noise_amount %f\n",
	   scalefactor, noise_amount);
  }

  virtual ~Noisetexture(void) {
  }
};


