/* userinterface.h : Prototypes for the user-interface functions.
** 
** Chris Butcher, 15 July 1997
*/

// Routines to provide a simple, K******-like builder interface.
object bind(object x);
object place(transform t, object x);
object rotate(axis around, double angle, object x);
object rotate(axis from, axis to, double angle, object x);
object rotate(vector fwd, vector up, object x);
object shift(double xshift, double yshift, double zshift, object x);
object shift(vector shift_vec, object x);
object stretch(double xstretch, double ystretch, double zstretch, object x);
object stretch(vector stretchvec, object x);
object reflect(axis the_axis, object x);
object scale(double scalefac, object x);
object shear(axis from, axis to, double amount, object x);
object set_label(char *labelname, object x);
object give_material(Material *m, object x);

// Routines to provide new primitive objects.
void init_primitives(void);
void deallocate_primitives(void);
object new_full(void);
object new_nothing(void);
object new_sphere(void);
object new_cyl(void);
object new_plane(void);
object new_cone(void);
object new_cone(double radius_1, double radius_2, double height);
object new_cube(void);
object new_torus(double r);
object new_torus(double circle_r, double tube_r);
object new_taper(double t, double r1, double r2);
object new_taper(double t, double r1, double r2, int steps, double limit);
object new_taper(double circle_r, double theta, double bottom_r, double top_r);
object new_unitcyl(void);
object new_stupid_cluster(object obj);
object new_uniform_cluster(object obj, vector min, vector max, int level,
			   int stop_at_first);
object new_uniform_cluster(object obj,
			   double x_min, double y_min, double z_min,
			   double x_max, double y_max, double z_max,
			   int level, int stop_at_first);
object new_octree_cluster(object obj, vector min, vector max, int limit,
			  int max_prims, int stop_at_first);
object new_octree_cluster(object obj,
			  double x_min, double y_min, double z_min,
			  double x_max, double y_max, double z_max,
			  int limit, int max_prims, int stop_at_first);
object new_heightfield(int smooth_normals, int should_stop, int do_jitter,
		       char *filename);
object new_heightfield(int smooth_normals, int should_stop,
		       int x_size, int y_size, height_val *field_data);
object new_volume(int smooth_normals, int stop_at_first,
		  int xres, int yres, int zres,
		  isosurface_val *field_data, isosurface_val threshold);
object new_point_light(double r, double g, double b);
object new_directional_light(double r, double g, double b,
			     double x, double y, double z);
object new_ambient_light(double r, double g, double b);
object new_limited_light(double r, double g, double b,
			 double limit, dropoff_type light_type);
object new_camera(OpticsModel *anOptics, RayTracer *anEngine,
		  PictureSampler *aSampler, Storage *aStorage,
		  PostProcessor *aProcess);
object new_extrusion(char *filename, int stop_at_first);
object new_revolution(int x_res, int y_res, int z_res,
		      double t_min, double t_max,
		      double (*func)(double tval), int should_stop);
object new_bilinear_patch(double x, double y, int flat);
object new_bilinear_patch(double x, double y, int flat, Cutout *cut);
object new_two_sided_patch(double x, double y, int flat);
object new_two_sided_patch(double x, double y, int flat, Cutout *cut);
object new_trunc_cone(double z);
object new_trunc_cone(double base, double end, double length);
object new_unitcone(void);
