/* xwindows.h : Header file for the XDisplay class.
** 
** This file defines the headers and definitions for the X display subsystem.
**
** Chris Butcher, 21 January 1997
*/

#ifdef XWINDOWS_ON
// All of the following code deals with the X11 interface. If this
// is not working on your system, you may wish to remove it by commenting
// out the XWINDOWS_ON definition in mirage.h.

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

class XDisplay : public DeletableObject {
public:
  // Variables to describe the X window/display setup
  Display *theXDisplay;
  Window theXWindow;
  GC theXGraphicsContext;
  int win_width, win_height;
  XImage *theXImage;
  Visual *theXVisual;
  int num_bitplanes;

  // Variables to describe the color packing setup in the XImage.
  unsigned long redmask, grnmask, blumask;
  int redshift, grnshift, blushift;
  int bits_per_pixel, bytes_per_pixel;
  int bytes_per_line, byte_pack_order;

  // Constructor performs X initialisation also.
  XDisplay(int img_width, int img_height, Display *use_display);

  // Draw a block of colour (x_min -> x_max - 1 and similarly in Y).
  void colour_block(int x_min, int y_min, int x_max, int y_max,
		    int r, int g, int b);

  // Set a pixel's colour.
  void colour_pixel(int pix_x, int pix_y, int r, int g, int b);

  // Check for Expose events that need to be handled.
  void check_exposed(void);

  // Initialise and allocate an XImage for use.
  XImage *initialise_XImage(int width, int height);

  // Destructor performs X cleanup also. This is called by the
  // DisplayAntialiaser after it has finished; it doesn't use the deletionlist.
  ~XDisplay(void) {
    free(theXImage->data);
    XFree(theXImage);
  }
};

#endif
